@@ -66,6 +66,8 @@ static const char *type2string(enum connman_network_type type)
 		break;
 	case CONNMAN_NETWORK_TYPE_WIFI:
 		return "wifi";
+	case CONNMAN_NETWORK_TYPE_WIMAX:
+		return "wimax";
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 		return "bluetooth";
@@ -158,6 +160,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 	struct connman_network *network = data;
 	DBusMessageIter iter, value;
 	const char *name;
+	int type;
 
 	DBG("conn %p", conn);
 
@@ -172,16 +175,26 @@ static DBusMessage *set_property(DBusConnection *conn,
 					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
 		return __connman_error_permission_denied(msg);
 
+	type = dbus_message_iter_get_arg_type(&value);
+
 	if (g_str_equal(name, "Remember") == TRUE) {
 		connman_bool_t remember;
 
+		if (type != DBUS_TYPE_BOOLEAN)
+			return __connman_error_invalid_arguments(msg);
+
 		dbus_message_iter_get_basic(&value, &remember);
 
 		if (network->remember == remember)
 			return __connman_error_invalid_arguments(msg);
+
+		network->remember = remember;
 	} else if (g_str_equal(name, "WiFi.Passphrase") == TRUE) {
 		const char *passphrase;
 
+		if (type != DBUS_TYPE_STRING)
+			return __connman_error_invalid_arguments(msg);
+
 		if (__connman_security_check_privilege(msg,
 					CONNMAN_SECURITY_PRIVILEGE_SECRET) < 0)
 			return __connman_error_permission_denied(msg);
@@ -193,6 +206,9 @@ static DBusMessage *set_property(DBusConnection *conn,
 	} else if (g_str_equal(name, "Priority") == TRUE) {
 		connman_uint8_t priority;
 
+		if (type != DBUS_TYPE_BYTE)
+			return __connman_error_invalid_arguments(msg);
+
 		dbus_message_iter_get_basic(&value, &priority);
 
 		network->priority = priority;
@@ -219,6 +235,12 @@ static DBusMessage *do_connect(DBusConnection *conn,
 		return __connman_error_failed(msg);
 
 	if (network->driver && network->driver->connect) {
+		enum connman_device_mode mode;
+
+		mode = connman_device_get_mode(network->device);
+		if (mode == CONNMAN_DEVICE_MODE_NETWORK_SINGLE)
+			__connman_device_disconnect(network->device);
+
 		err = network->driver->connect(network);
 		if (err < 0 && err != -EINPROGRESS)
 			return __connman_error_failed(msg);
@@ -245,6 +267,8 @@ static DBusMessage *do_disconnect(DBusConnection *conn,
 
 	connman_element_unregister_children(&network->element);
 
+	connman_device_set_disconnected(network->device, TRUE);
+
 	if (network->driver && network->driver->disconnect) {
 		err = network->driver->disconnect(network);
 		if (err < 0 && err != -EINPROGRESS)
@@ -423,6 +447,8 @@ struct connman_network *connman_network_create(const char *identifier,
 	struct connman_network *network;
 	connman_uint8_t strength = 0;
 	const char *str;
+	char *temp;
+	unsigned int i;
 
 	DBG("identifier %s type %d", identifier, type);
 
@@ -432,21 +458,33 @@ struct connman_network *connman_network_create(const char *identifier,
 
 	DBG("network %p", network);
 
-	network->element.refcount = 1;
+	__connman_element_initialize(&network->element);
 
-	network->element.name = g_strdup(identifier);
+	temp = g_strdup(identifier);
+	if (temp == NULL) {
+		g_free(network);
+		return NULL;
+	}
+
+	for (i = 0; i < strlen(temp); i++) {
+		char tmp = temp[i];
+		if ((tmp < '0' || tmp > '9') && (tmp < 'A' || tmp > 'Z') &&
+						(tmp < 'a' || tmp > 'z'))
+			temp[i] = '_';
+	}
+
+	network->element.name = temp;
 	network->element.type = CONNMAN_ELEMENT_TYPE_NETWORK;
-	network->element.index = -1;
 
 	network->element.network = network;
 	network->element.destruct = network_destruct;
 
 	str = type2string(type);
 	if (str != NULL)
-		connman_element_add_static_property(&network->element,
+		connman_element_set_static_property(&network->element,
 					"Type", DBUS_TYPE_STRING, &str);
 
-	connman_element_add_static_property(&network->element,
+	connman_element_set_static_property(&network->element,
 					"Strength", DBUS_TYPE_BYTE, &strength);
 
 	network->type = type;
@@ -562,7 +600,10 @@ int connman_network_set_available(struct connman_network *network,
 	if (network->registered == FALSE)
 		return 0;
 
-	if (network->connected == FALSE && network->remember == FALSE)
+	if (network->connected == TRUE)
+		return 0;
+
+	if (network->remember == FALSE)
 		return 0;
 
 	signal = dbus_message_new_signal(network->element.path,
@@ -595,6 +636,51 @@ connman_bool_t connman_network_get_available(struct connman_network *network)
 	return network->available;
 }
 
+static gboolean set_connected(gpointer user_data)
+{
+	struct connman_network *network = user_data;
+
+	if (network->connected == TRUE) {
+		struct connman_element *element;
+		enum connman_element_type type = CONNMAN_ELEMENT_TYPE_UNKNOWN;
+
+		switch (network->protocol) {
+		case CONNMAN_NETWORK_PROTOCOL_UNKNOWN:
+			return 0;
+		case CONNMAN_NETWORK_PROTOCOL_IP:
+			type = CONNMAN_ELEMENT_TYPE_DHCP;
+			break;
+		case CONNMAN_NETWORK_PROTOCOL_PPP:
+			type = CONNMAN_ELEMENT_TYPE_PPP;
+			break;
+		}
+
+		__connman_device_increase_connections(network->device);
+
+		__connman_device_set_network(network->device, network);
+
+		connman_device_set_disconnected(network->device, FALSE);
+
+		element = connman_element_create(NULL);
+		if (element != NULL) {
+			element->type  = type;
+			element->index = network->element.index;
+
+			if (connman_element_register(element,
+						&network->element) < 0)
+				connman_element_unref(element);
+		}
+	} else {
+		connman_element_unregister_children(&network->element);
+
+		__connman_device_set_network(network->device, NULL);
+
+		__connman_device_decrease_connections(network->device);
+	}
+
+	return FALSE;
+}
+
 /**
  * connman_network_set_connected:
  * @network: network structure
@@ -616,8 +702,10 @@ int connman_network_set_connected(struct connman_network *network,
 
 	network->connected = connected;
 
-	if (network->registered == FALSE)
+	if (network->registered == FALSE) {
+		g_idle_add(set_connected, network);
 		return 0;
+	}
 
 	signal = dbus_message_new_signal(network->element.path,
 				CONNMAN_NETWORK_INTERFACE, "PropertyChanged");
@@ -635,41 +723,22 @@ int connman_network_set_connected(struct connman_network *network,
 
 	g_dbus_send_message(connection, signal);
 
-	if (connected == TRUE) {
-		struct connman_element *element;
-		enum connman_element_type type = CONNMAN_ELEMENT_TYPE_UNKNOWN;
-
-		switch (network->protocol) {
-		case CONNMAN_NETWORK_PROTOCOL_UNKNOWN:
-			return 0;
-		case CONNMAN_NETWORK_PROTOCOL_IP:
-			type = CONNMAN_ELEMENT_TYPE_DHCP;
-			break;
-		case CONNMAN_NETWORK_PROTOCOL_PPP:
-			type = CONNMAN_ELEMENT_TYPE_PPP;
-			break;
-		}
-
-		__connman_device_increase_connections(network->device);
-
-		element = connman_element_create(NULL);
-		if (element != NULL) {
-			element->type  = type;
-			element->index = network->element.index;
-
-			if (connman_element_register(element,
-						&network->element) < 0)
-				connman_element_unref(element);
-		}
-	} else {
-		connman_element_unregister_children(&network->element);
-
-		__connman_device_decrease_connections(network->device);
-	}
+	set_connected(network);
 
 	return 0;
 }
 
+/**
+ * connman_network_get_connected:
+ * @network: network structure
+ *
+ * Get network connection status
+ */
+connman_bool_t connman_network_get_connected(struct connman_network *network)
+{
+	return network->connected;
+}
+
 /**
  * connman_network_set_remember:
  * @network: network structure
@@ -743,6 +812,19 @@ int connman_network_connect(struct connman_network *network)
 	return 0;
 }
 
+int __connman_network_disconnect(struct connman_network *network)
+{
+	if (network->connected == FALSE)
+		return -ENOTCONN;
+
+	if (network->driver && network->driver->disconnect)
+		return network->driver->disconnect(network);
+
+	network->connected = FALSE;
+
+	return 0;
+}
+
 /**
  * connman_network_set_string:
  * @network: network structure
@@ -1059,15 +1141,13 @@ static int network_load(struct connman_network *network)
 	if (val > 0)
 		network->priority = val;
 
-	if (network->remember == TRUE) {
-		g_free(network->wifi.security);
-		network->wifi.security = g_key_file_get_string(keyfile,
+	g_free(network->wifi.security);
+	network->wifi.security = g_key_file_get_string(keyfile,
 				network->identifier, "WiFi.Security", NULL);
 
-		g_free(network->wifi.passphrase);
-		network->wifi.passphrase = g_key_file_get_string(keyfile,
+	g_free(network->wifi.passphrase);
+	network->wifi.passphrase = g_key_file_get_string(keyfile,
 				network->identifier, "WiFi.Passphrase", NULL);
-	}
 
 	g_key_file_free(keyfile);
 
@@ -1112,7 +1192,7 @@ update:
 		g_key_file_set_integer(keyfile, network->identifier,
 						"Priority", network->priority);
 
-	if (network->remember == TRUE) {
+	if (network->remember == TRUE || network->connected == TRUE) {
 		if (network->wifi.security != NULL)
 			g_key_file_set_string(keyfile, network->identifier,
 				"WiFi.Security", network->wifi.security);
